@@ -1,13 +1,14 @@
 ;;;; xslide.el --- XSL Integrated Development Environment
-;; $Id: xslide.el,v 1.2 2001/09/05 21:39:04 menteith Exp $
+;; $Id: xslide.el,v 1.11 2003/07/18 23:27:13 tonygraham Exp $
 
-;; Copyright (C) 1998, 1999, 2000, 2001 Tony Graham
+;; Copyright (C) 1998, 1999, 2000, 2001, 2003 Tony Graham
 
 ;; Author: Tony Graham <tkg@menteith.com>
 ;; Contributors: Simon Brooke, Girard Milmeister, Norman Walsh,
-;;               Moritz Maass, Lassi Tuura, Simon Wright, KURODA Akira
+;;               Moritz Maass, Lassi Tuura, Simon Wright, KURODA Akira,
+;;               Ville Skytt, Glen Peterson
 ;; Created: 21 August 1998
-;; Version: $Revision: 1.2 $
+;; Version: $Revision: 1.11 $
 ;; Keywords: languages, xsl, xml
 
 ;;; This file is not part of GNU Emacs.
@@ -59,6 +60,16 @@
 (require 'xslide-font "xslide-font")
 (require 'xslide-process "xslide-process")
 
+;; Work out if using XEmacs or Emacs
+;; Inspired by 'vm'
+(defconst xsl-xemacs-p nil)
+(defconst xsl-fsfemacs-p nil)
+(defun xsl-xemacs-p () xsl-xemacs-p)
+(defun xsl-fsfemacs-p () xsl-fsfemacs-p)
+(defun xsl-note-emacs-version ()
+  (setq xsl-xemacs-p (string-match "XEmacs" emacs-version)
+	xsl-fsfemacs-p (not xsl-xemacs-p)))
+(xsl-note-emacs-version)
 
 ;; Define core `xsl' group.
 (defgroup xsl nil
@@ -78,11 +89,11 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Version information
 
-(defconst xslide-version "0.2"
+(defconst xslide-version "0.2.2"
   "Version number of xslide XSL mode.")
 
 (defun xslide-version ()
-  "Returns the value of the variable xslide-version."
+  "Returns the value of the variable `xslide-version'."
   xslide-version)
 
 (defconst xslide-maintainer-address "xslide-bug@menteith.com")
@@ -91,10 +102,10 @@
 ;; Variables
 
 (defvar xsl-indent-tabs-mode nil
-  "*Initial value of indent-tabs-mode on entering xsl-mode")
+  "*Initial value of indent-tabs-mode on entering `xsl-mode'.")
 
 (defvar xsl-default-filespec "*.xsl"
-  "*Inital prompt value for `xsl-etags''s FILESPEC argument.")
+  "*Initial prompt value for `xsl-etags''s FILESPEC argument.")
 
 (defvar xsl-filespec-history (list xsl-default-filespec)
   "Minibuffer history list for `xsl-etags' and `xsl-grep''s FILESPEC argument.")
@@ -106,16 +117,16 @@
   "*Non-nil disables case insensitive searches by `xsl-grep'.")
 
 (defvar xsl-comment-start "<!--"
-  "*Comment start character sequence")
+  "*Comment start character sequence.")
 
 (defvar xsl-comment-end "-->"
-  "*Comment end character sequence")
+  "*Comment end character sequence.")
 
 (defvar xsl-comment-max-column 70
-  "*Maximum column number for text in a comment")
+  "*Maximum column number for text in a comment.")
 
 (defcustom xsl-initial-stylesheet-file (locate-library "xslide-initial.xsl" t)
-  "*File containing initial stylesheet inserted into empty XSL buffers"
+  "*File containing initial stylesheet inserted into empty XSL buffers."
   :type '(choice (file :must-match t) (const :tag "No initial stylesheet" nil))
   :group 'xsl)
 
@@ -124,6 +135,16 @@
   :type '(integer)
   :group 'xsl)
 
+(defcustom xsl-initial-fo-file (locate-library "xslide-initial.fo" t)
+  "*File containing initial FO stylesheet inserted into empty XSL buffers."
+  :type '(choice (file :must-match t) (const :tag "No initial FO file" nil))
+  :group 'xsl)
+
+(defcustom xsl-initial-fo-initial-point 0
+  "*Initial position of point in initial FO stylesheet."
+  :type '(integer)
+  :group 'xsl)
+
 (defcustom xsl-indent-attributes nil
   "*Whether to indent attributes on lines following an open tag.
 If non-nil, attributes will be aligned with the space after the
@@ -220,6 +241,7 @@ lines above and below COMMENT in the manner of `xsl-big-comment'."
   (define-key xsl-mode-map [(control o)]
 				   	  'xsl-open-line)
   (define-key xsl-mode-map "\C-c<"  	  'xsl-insert-tag)
+  (define-key xsl-mode-map "\C-c\C-t"     'xsl-if-to-choose)
 ;;  (define-key xsl-mode-map [(control m)]	  'xsl-electric-return)
 ;;  (define-key xsl-mode-map \10	  'xsl-electric-return)
   (define-key xsl-mode-map "\177"	  'backward-delete-char-untabify)
@@ -228,6 +250,82 @@ lines above and below COMMENT in the manner of `xsl-big-comment'."
 ;;  (define-key xsl-mode-map "\M-\C-h" 'mark-xsl-rule)
 )
 
+(defun xsl-if-to-choose ()
+"Converts <xsl:if> to <xsl:choose>.  Works on a single 'ifs' or on a region.
+So:
+
+<xsl:if test=\"isFive = 5\"><p>It's five!</p></xsl:if>
+
+Becomes:
+<xsl:choose>
+   <xsl:when test=\"isFive = 5\"><p>It's five!</p></xsl:when>
+   <xsl:otherwise></xsl:otherwise>
+</xsl:choose>
+
+If you put your cursor inside the open-tag of the if, it will work on that tag
+only.  If you highlight a region, it will convert every 'if' whose start tag is
+within that region.  It is very easy to convert consecutive 'if's to a single
+choose by deleting the appropriate lines after executing this command.
+
+Bound to C-c C-t by default."
+  (interactive)
+  (let
+    (
+      (single-if (not (mark)))
+      (the-start (point))
+      (the-end (if (mark) (mark) (point)))
+    )
+    (if (and (not (null (mark)))
+             (< (mark) (point)))
+      (progn
+        (exchange-point-and-mark)
+        (setq the-start (point))
+        (setq the-end (mark))
+      )
+    )
+    (save-excursion
+      (if single-if
+        (progn
+          (search-backward "<" nil t)
+;          (message "xsl-if-to-choose: single if mode")
+          (xsl-convert-if-to-choose-slave)
+        )
+        (save-excursion
+;          (message
+;            (concat "xsl-if-to-choose: Region mode: "
+;                    (int-to-string the-start)
+;                    " "
+;                    (int-to-string the-end)
+;            )
+;          )
+          (goto-char the-end)
+          (if (save-excursion (search-backward "<xsl:if" the-start t))
+            (while (search-backward "<xsl:if" the-start t)
+              (xsl-convert-if-to-choose-slave)
+            )
+            (message "xsl-if-to-choose error: There's no <xsl:if> within the selected region.")            
+          )
+        )
+      )
+    )
+  )
+)
+
+(defun xsl-convert-if-to-choose-slave ()
+  (if (looking-at "<xsl:if")
+    (let
+      ( (start (save-excursion (beginning-of-line) (point))) )      
+      (delete-char 7)
+      (insert "<xsl:choose>\n<xsl:when")
+      (search-forward "</xsl:if>" nil t)
+      (backward-delete-char 9)
+      (insert "</xsl:when>\n<xsl:otherwise></xsl:otherwise>\n</xsl:choose>")
+      (indent-region start (point) nil)
+    )
+      (message "xsl-if-to-choose error: point is not within the start tag of an <xsl:if>.")
+  )
+)
+
 (defun xsl-electric-greater-than (arg)
   "Insert a \">\" and, optionally, insert a matching end-tag.
 
@@ -276,7 +374,7 @@ current line."
 	 (xsl-font-lock-region-point-max)))))
 
 (defun xsl-electric-apos ()
-  "Function called when \"'\" is pressed in XSL mode"
+  "Function called when \"'\" is pressed in XSL mode."
   (interactive)
   (insert "'")
   (if (looking-at "\\([\"/})]\\|$\\)")
@@ -284,7 +382,7 @@ current line."
 	(insert "'"))))
 
 (defun xsl-electric-quote ()
-  "Function called when '\"' is pressed in XSL mode"
+  "Function called when '\"' is pressed in XSL mode."
   (interactive)
   (insert "\"")
   (if (looking-at "\\(['/})]\\|$\\)")
@@ -292,7 +390,7 @@ current line."
 	(insert "\""))))
 
 (defun xsl-electric-lsqb ()
-  "Function called when \"[\" is pressed in XSL mode"
+  "Function called when \"[\" is pressed in XSL mode."
   (interactive)
   (insert "[")
   (if (looking-at "\\([\"'/})]\\|$\\)")
@@ -300,7 +398,7 @@ current line."
 	(insert "]"))))
 
 (defun xsl-electric-lpar ()
-  "Function called when \"(\" is pressed in XSL mode"
+  "Function called when \"(\" is pressed in XSL mode."
   (interactive)
   (insert "(")
   (if (looking-at "\\([\]\"'/}]\\|$\\)")
@@ -308,7 +406,7 @@ current line."
 	(insert ")"))))
 
 (defun xsl-electric-lcub ()
-  "Function called when \"{\" is pressed in XSL mode"
+  "Function called when \"{\" is pressed in XSL mode."
   (interactive)
   (insert "{")
   (if (looking-at "\\([\])\"'/}]\\|$\\)")
@@ -316,13 +414,13 @@ current line."
 	(insert "}"))))
 
 (defun xsl-electric-less-than ()
-  "Function called when \"<\" is pressed in XSL mode"
+  "Function called when \"<\" is pressed in XSL mode."
   (interactive)
   (insert "<")
   (xsl-electric-tab))
 
 (defun xsl-match-opening-tag (a)
-  "Function called to match the next opening tag to a closing tag"
+  "Function called to match the next opening tag to a closing tag."
   (if (looking-at "</")
       (catch 'start-tag
         (while (re-search-backward
@@ -336,7 +434,7 @@ current line."
     nil)
 )
 (defun xsl-electric-slash ()
-  "Function called when \"/\" is pressed in XSL mode"
+  "Function called when \"/\" is pressed in XSL mode."
   (interactive)
   (insert "/")
   (xsl-electric-tab)
@@ -377,6 +475,7 @@ current line."
 		     (xsl-font-lock-region-point-max)))))))))
 
 (defun xsl-electric-return ()
+  "Function called when RET is pressed in XSL mode."
   (interactive)
   (insert "\n")
   (xsl-electric-tab))
@@ -413,7 +512,7 @@ current line."
 
 
 (defun xsl-close-open-tab-p nil
-  "Return 'T' if the current line contains more right than left angle-brackets"
+  "Return t if the current line contains more right than left angle-brackets."
   (save-excursion
     (beginning-of-line)
     (let ((open 0))
@@ -433,8 +532,7 @@ current line."
   )
 
 (defun xsl-calculate-indent ()
-  "Calculate what the indent should be for the current line"
-  (interactive)
+  "Calculate what the indent should be for the current line."
   (let* ((limit   (point))
  	 (name    "[^<>=\"' \t\n]+")
  	 (string  "\\(\"[^<>\"]*\"\\|'[^<>']*'\\)")
@@ -442,7 +540,7 @@ current line."
  	 (attval  (concat name "=" string))
  	 (oattval (concat name "=" ostring))
  	 (element (concat "<\\(" name "\\)"
- 			  "\\([ \t\n]+" attval "\\)*"))
+ 			  "\\([ \t\n]+" attval "\\)*[ \t\n]*"))
  	 (meta    (concat "<!\\(DOCTYPE\\|ENTITY\\)"
  			  "\\([ \t\n]+\\(" name "\\|" string "\\)\\)*")))
     (save-excursion
@@ -553,15 +651,15 @@ otherwise complete with allowed attributes."
 	       (insert completion))
 	      (t
 	       (goto-char here)
-	       (let ((res (completing-read
-			   "xsl-complete: " tab nil t pattern)))
-		 (insert
-		  ;; insert completed strings(res) after pattern
-		  (substring res (length pattern))))))))))
+	       (message "Making completion list...")
+	       (let ((list (all-completions pattern tab)))
+		 (with-output-to-temp-buffer " *Completions*"
+		   (display-completion-list list)))
+	       (message "Making completion list...%s" "done")))))))
 
 (defun xsl-insert-tag (tag)
   "Insert a tag, reading tag name in minibuffer with completion."
-  (interactive 
+  (interactive
    (list
     (completing-read "Tag: " xsl-all-elements-alist)))
   ;;  (xsl-find-context-of (point))
@@ -613,7 +711,7 @@ otherwise complete with allowed attributes."
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (defun xsl-insert-template (match)
-  "Insert a template"
+  "Insert a template."
   (interactive "smatch=")
   (xsl-electric-tab)
   (insert (format "<xsl:template match=\"%s\">\n" match))
@@ -676,13 +774,13 @@ otherwise complete with allowed attributes."
 ;; imenu stuff
 
 (defun xsl-sort-alist (alist)
-  "Sort an alist"
+  "Sort an alist."
   (sort
    alist
    (lambda (a b) (string< (car a) (car b)))))
 
 (defun xsl-imenu-create-index-function ()
-  "Create an alist of elements, etc. suitable for use with imenu."
+  "Create an alist of elements, etc. suitable for use with `imenu'."
   (interactive)
   (let ((template-alist '())
 	(mode-alist '())
@@ -884,8 +982,7 @@ When called interactively, the initial FILESPEC is taken from
 xsl-default-filespec, but `xsl-grep' also maintains a history of
 FILESPEC arguments so you can easily re-use a previous value.  The
 history is shared with `xsl-etags' so you can re-use the same FILESPEC
-with both functions.
-"
+with both functions."
   (interactive
    (list
     (xsl-read-from-minibuffer "Pattern: "
@@ -918,8 +1015,8 @@ preloaded into the abbreviations table.
 
 Font lock mode:
 
-Turning on font lock mode causes various XSL syntactic structures to be 
-hightlighted. To turn this on whenever you visit an XSL file, add
+Turning on font lock mode causes various XSL syntactic structures to be
+highlighted. To turn this on whenever you visit an XSL file, add
 the following to your .emacs file:
   \(add-hook 'xsl-mode-hook 'turn-on-font-lock\)
 
@@ -980,9 +1077,15 @@ the prompts.
 ;;  (xsl-font-make-faces)
   (make-local-variable 'font-lock-defaults)
   (setq font-lock-defaults '(xsl-font-lock-keywords t))
-  (setq font-lock-mark-block-function 'xsl-font-lock-mark-block-function)
+  (if (xsl-fsfemacs-p)
+      (progn
+	(make-local-variable 'font-lock-mark-block-function)
+	(setq font-lock-mark-block-function
+	      'xsl-font-lock-mark-block-function)))
+  (make-local-variable 'indent-line-function)
+  (setq indent-line-function `xsl-electric-tab)
 ;;  (make-local-variable 'font-lock-defaults)
-;;  (setq font-lock-defaults 
+;;  (setq font-lock-defaults
 ;;	'(xsl-font-lock-keywords nil t ((?- . "w")
 ;;					(?_ . "w")
 ;;					(?. . "w"))))
@@ -1007,17 +1110,15 @@ the prompts.
 
 ;;;; Bug reporting
 
-(eval-and-compile
-  (autoload 'reporter-submit-bug-report "reporter"))
-
 (defun xsl-submit-bug-report ()
-  "Submit via mail a bug report on XSLIDE."
+  "Submit via mail a bug report on 'xslide'."
   (interactive)
+  (require 'reporter)
   (and (y-or-n-p "Do you really want to submit a report on XSL mode? ")
        (reporter-submit-bug-report
 	xslide-maintainer-address
 	(concat "xslide.el " xslide-version)
-	(list 
+	(list
 	 )
 	nil
 	nil
@@ -1028,9 +1129,11 @@ the prompts.
       (beginning-of-line)
       (delete-region (point) (progn (forward-line) (point)))
       (insert
-       "Subject: XSLIDE version " xslide-version " is wonderful but...\n"))))
+       "Subject: xslide " xslide-version " is wonderful but...\n"))))
 
 
+(autoload 'reporter-submit-bug-report "reporter")
+
 ;;;; Last provisions
 ;;;(provide 'xslide)
 
